@@ -37,6 +37,7 @@
 #include <gtkmm/button.h>
 #include <gtkmm/stock.h>
 
+#include "gnote.hpp"
 #include "note.hpp"
 #include "notemanager.hpp"
 #include "noterenamedialog.hpp"
@@ -64,19 +65,19 @@ namespace gnote {
     {
       std::string message;
 
-      // %1% is the number of note.
-      message = ngettext("Really delete this note?", "Really delete these %1% notes?", notes.size());
-
-      boost::format format(message);
-      // make sure an exception is not raised for the non-plural form
-      format.exceptions( boost::io::all_error_bits ^ 
-                         ( boost::io::too_many_args_bit | 
-                           boost::io::too_few_args_bit )  );
+      if(notes.size() == 1) {
+        // TRANSLATORS: %1% will be replaced by note title
+        message = str(boost::format("Really delete \"%1%\"?") % notes.front()->get_title());
+      }
+      else {
+        // TRANSLATORS: %1% is number of notes
+        message = str(boost::format(ngettext("Really delete %1% note?", "Really delete %1% notes?", notes.size())) % notes.size());
+      }
 
       utils::HIGMessageDialog dialog(parent, GTK_DIALOG_DESTROY_WITH_PARENT,
                                      Gtk::MESSAGE_QUESTION,
                                      Gtk::BUTTONS_NONE,
-                                     str(format % notes.size()),
+                                     message,
                                      _("If you delete a note it is permanently lost."));
 
       Gtk::Button *button;
@@ -124,141 +125,13 @@ namespace gnote {
 
   }
 
-  class NoteData
-  {
-  public:
-    typedef std::map<std::string, Tag::Ptr> TagMap;
-    NoteData(const std::string & _uri);
-
-    const std::string & uri() const
-      {
-        return m_uri;
-      }
-    const std::string & title() const
-      {
-        return m_title;
-      }
-    std::string & title()
-      {
-        return m_title;
-      }
-    const std::string & text() const
-      { 
-        return m_text;
-      }
-    std::string & text()
-      { 
-        return m_text;
-      }
-    const sharp::DateTime & create_date() const
-      {
-        return m_create_date;
-      }
-    sharp::DateTime & create_date()
-      {
-        return m_create_date;
-      }
-    const sharp::DateTime & change_date() const
-      {
-        return m_change_date;
-      }
-    void set_change_date(const sharp::DateTime & date)
-      {
-        m_change_date = date;
-        m_metadata_change_date = date;
-      }
-    const sharp::DateTime & metadata_change_date() const
-      {
-        return m_metadata_change_date;
-      }
-    sharp::DateTime & metadata_change_date()
-      {
-        return m_metadata_change_date;
-      }
-    int cursor_position() const
-      {
-        return m_cursor_pos;
-      }
-    void set_cursor_position(int new_pos)
-      {
-        m_cursor_pos = new_pos;
-      }
-    int width() const
-      {
-        return m_width;
-      }
-    int & width()
-      {
-        return m_width;
-      }
-    int height() const
-      {
-        return m_height;
-      }
-    int & height()
-      {
-        return m_height;
-      }
-    int x() const
-      {
-        return m_x;
-      }
-    int & x()
-      {
-        return m_x;
-      }
-    int y() const
-      {
-        return m_y;
-      }
-    int & y()
-      {
-        return m_y;
-      }
-    const TagMap & tags() const
-      {
-        return m_tags;
-      }
-    TagMap & tags()
-      {
-        return m_tags;
-      }
-    
-    bool is_open_on_startup() const
-      {
-        return m_open_on_startup;
-      }
-    void set_is_open_on_startup(bool v)
-      {
-        m_open_on_startup = v;
-      }
-    void set_position_extent(int x, int y, int width, int height);
-    bool has_position();
-    bool has_extent();
-
-  private:
-    const std::string m_uri;
-    std::string       m_title;
-    std::string       m_text;
-    sharp::DateTime             m_create_date;
-    sharp::DateTime             m_change_date;
-    sharp::DateTime             m_metadata_change_date;
-    int               m_cursor_pos;
-    int               m_width, m_height;
-    int               m_x, m_y;
-    bool              m_open_on_startup;
-
-    TagMap m_tags;
-  
-    static const int  s_noPosition;
-  };
-
 
   const int  NoteData::s_noPosition = -1;
 
   NoteData::NoteData(const std::string & _uri)
     : m_uri(_uri)
     , m_cursor_pos(0)
+    , m_selection_bound_pos(s_noPosition)
     , m_width(0)
     , m_height(0)
     , m_x(s_noPosition)
@@ -364,6 +237,13 @@ namespace gnote {
       }
       m_buffer->place_cursor(cursor);
 
+      if(m_data->selection_bound_position() >= 0) {
+        // Move selection bound to last-saved position
+        Gtk::TextIter selection_bound;
+        selection_bound = m_buffer->get_iter_at_offset(m_data->selection_bound_position());
+        m_buffer->move_mark(m_buffer->get_selection_bound(), selection_bound);
+      }
+
       // New events should create Undo actions
       m_buffer->undoer().thaw_undo ();
     }
@@ -551,7 +431,7 @@ namespace gnote {
   {
     if(NoteTagTable::tag_is_serializable(tag)) {
       DBG_OUT("BufferTagApplied queueing save: %s", tag->property_name().get_value().c_str());
-      queue_save(CONTENT_CHANGED);
+      queue_save(get_tag_table()->get_change_type(tag));
     }
   }
 
@@ -561,18 +441,24 @@ namespace gnote {
   {
     if(NoteTagTable::tag_is_serializable(tag)) {
       DBG_OUT("BufferTagRemoved queueing save: %s", tag->property_name().get_value().c_str());
-      queue_save(CONTENT_CHANGED);
+      queue_save(get_tag_table()->get_change_type(tag));
     }
   }
 
-  void Note::on_buffer_insert_mark_set(const Gtk::TextBuffer::iterator & iter,
-                                       const Glib::RefPtr<Gtk::TextBuffer::Mark> & insert)
+  void Note::on_buffer_mark_set(const Gtk::TextBuffer::iterator & iter,
+                                const Glib::RefPtr<Gtk::TextBuffer::Mark> & insert)
   {
-    if(insert != m_buffer->get_insert()) {
+    if(insert == m_buffer->get_insert()) {
+      m_data.data().set_cursor_position(iter.get_offset());
+    }
+    else if(insert == m_buffer->get_selection_bound()) {
+      m_data.data().set_selection_bound_position(iter.get_offset());
+    }
+    else {
       return;
     }
-    m_data.data().set_cursor_position(iter.get_offset());
-    DBG_OUT("BufferInsertSetMark queueing save");
+
+    DBG_OUT("OnBufferSetMark queueing save");
     queue_save(NO_CHANGE);
   }
 
@@ -800,10 +686,9 @@ namespace gnote {
     }
 
     if (!linking_notes.empty()) {
-      Preferences & preferences = Preferences::obj();
+      Glib::RefPtr<Gio::Settings> settings = Preferences::obj().get_schema_settings(Preferences::SCHEMA_GNOTE);
       const NoteRenameBehavior behavior
-        = static_cast<NoteRenameBehavior>(
-            preferences.get<int>(Preferences::NOTE_RENAME_BEHAVIOR));
+        = static_cast<NoteRenameBehavior>(settings->get_int(Preferences::NOTE_RENAME_BEHAVIOR));
 
       if (NOTE_RENAME_ALWAYS_SHOW_DIALOG == behavior) {
         NoteRenameDialog dlg(linking_notes, old_title, self);
@@ -813,8 +698,7 @@ namespace gnote {
         if (Gtk::RESPONSE_CANCEL != response
             && NOTE_RENAME_ALWAYS_SHOW_DIALOG
                  != selected_behavior) {
-          preferences.set<int>(Preferences::NOTE_RENAME_BEHAVIOR,
-                               selected_behavior);
+          settings->set_int(Preferences::NOTE_RENAME_BEHAVIOR, selected_behavior);
         }
 
         const NoteRenameDialog::MapPtr notes = dlg.get_notes();
@@ -1105,7 +989,7 @@ namespace gnote {
 
   const Glib::RefPtr<NoteTagTable> & Note::get_tag_table()
   {
-    if (m_tag_table == NULL) {
+    if (!m_tag_table) {
       // NOTE: Sharing the same TagTable means
       // that formatting is duplicated between
       // buffers.
@@ -1128,7 +1012,7 @@ namespace gnote {
       m_buffer->signal_remove_tag().connect(
         sigc::mem_fun(*this, &Note::on_buffer_tag_removed));
       m_buffer->signal_mark_set().connect(
-        sigc::mem_fun(*this, &Note::on_buffer_insert_mark_set));
+        sigc::mem_fun(*this, &Note::on_buffer_mark_set));
     }
     return m_buffer;
   }
@@ -1142,6 +1026,10 @@ namespace gnote {
         sigc::mem_fun(*this, &Note::on_window_destroyed));
       m_window->signal_configure_event().connect(
         sigc::mem_fun(*this, &Note::on_window_configure), false);
+      m_window->signal_show().connect(
+        sigc::mem_fun(*this, &Note::on_window_show));
+      m_window->signal_hide().connect(
+        sigc::mem_fun(*this, &Note::on_window_hide));
 
       if (m_data.data().has_extent())
         m_window->set_default_size(m_data.data().width(),
@@ -1161,6 +1049,17 @@ namespace gnote {
     return m_window;
   }
 
+  void Note::on_window_show()
+  {
+    Gnote::obj().add_window(m_window);
+  }
+
+
+  void Note::on_window_hide()
+  {
+    Gnote::obj().remove_window(m_window);
+  }
+
 
   bool Note::is_special() const
   { 
@@ -1175,7 +1074,8 @@ namespace gnote {
 
   bool Note::is_pinned() const
   {
-    std::string pinned_uris = Preferences::obj().get<std::string>(Preferences::MENU_PINNED_NOTES);
+    std::string pinned_uris = Preferences::obj()
+      .get_schema_settings(Preferences::SCHEMA_GNOTE)->get_string(Preferences::MENU_PINNED_NOTES);
     return (boost::find_first(pinned_uris, uri()));
   }
 
@@ -1183,8 +1083,8 @@ namespace gnote {
   void Note::set_pinned(bool pinned) const
   {
     std::string new_pinned;
-    // this is like a calle to is_pinned() but we want to reating the gconf value
-    std::string old_pinned = Preferences::obj().get<std::string>(Preferences::MENU_PINNED_NOTES);
+    Glib::RefPtr<Gio::Settings> settings = Preferences::obj().get_schema_settings(Preferences::SCHEMA_GNOTE);
+    std::string old_pinned = settings->get_string(Preferences::MENU_PINNED_NOTES);
     bool is_currently_pinned = (boost::find_first(old_pinned, uri()));
 
     if (pinned == is_currently_pinned)
@@ -1204,7 +1104,7 @@ namespace gnote {
         }
       }
     }
-    Preferences::obj().set<std::string>(Preferences::MENU_PINNED_NOTES, new_pinned);
+    settings->set_string(Preferences::MENU_PINNED_NOTES, new_pinned);
   }
 
   
@@ -1276,6 +1176,9 @@ namespace gnote {
         else if(name == "cursor-position") {
           note->set_cursor_position(boost::lexical_cast<int>(xml.read_string()));
         }
+        else if(name == "selection-bound-position") {
+          note->set_selection_bound_position(boost::lexical_cast<int>(xml.read_string()));
+        }
         else if(name == "width") {
           note->width() = boost::lexical_cast<int>(xml.read_string());
         }
@@ -1424,6 +1327,10 @@ namespace gnote {
     xml.write_string (boost::lexical_cast<std::string>(note.cursor_position()));
     xml.write_end_element ();
 
+    xml.write_start_element("", "selection-bound-position", "");
+    xml.write_string(boost::lexical_cast<std::string>(note.selection_bound_position()));
+    xml.write_end_element();
+
     xml.write_start_element ("", "width", "");
     xml.write_string (boost::lexical_cast<std::string>(note.width()));
     xml.write_end_element ();
