@@ -103,7 +103,7 @@ namespace gnote {
     notebook->show();
     
     notebook->append_page (*manage(make_editing_pane()),
-                           _("Editing"));
+                           _("General"));
     notebook->append_page (*manage(make_hotkeys_pane()),
                            _("Hotkeys"));
 //      }
@@ -211,6 +211,13 @@ namespace gnote {
       options_list->show();
 
 
+      // Status icon
+      check = manage(make_check_button(_("Use Status _Icon")));
+      options_list->pack_start(*check, false, false, 0);
+      peditor = new sharp::PropertyEditorBool(settings, Preferences::USE_STATUS_ICON, *check);
+      peditor->setup();
+
+
       // Spell checking...
 
 #if FIXED_GTKSPELL
