@@ -27,6 +27,7 @@
 
 #include <glibmm/i18n.h>
 #include <gtkmm/image.h>
+#include <gtkmm/linkbutton.h>
 #include <gtkmm/main.h>
 #include <gtkmm/stock.h>
 #include <gtkmm/table.h>
@@ -94,13 +95,14 @@ namespace gnote {
     , m_menubar(NULL)
     , m_find_combo(Glib::RefPtr<Gtk::TreeModel>::cast_static(Gtk::ListStore::create(m_find_combo_columns)), true)
     , m_clear_search_button(Gtk::Stock::CLEAR)
-    , m_case_sensitive(_("C_ase Sensitive"), true)
     , m_content_vbox(false, 0)
     , m_matches_column(NULL)
+    , m_no_matches_box(NULL)
     , m_tree(NULL)
     , m_entry_changed_timeout(NULL)
     , m_clickX(0), m_clickY(0)
   {
+    Gnote::obj().add_window(this);
     _init_static();
 //    get_window()->set_icon_name("gnote");
     set_default_size(450,400);
@@ -110,8 +112,6 @@ namespace gnote {
 
     m_menubar = create_menu_bar ();
 
-    Gtk::Image *image = manage(new Gtk::Image (utils::get_icon ("system-search", 48)));
-
     Gtk::Label *label = manage(new Gtk::Label (_("_Search:"), true));
     label->property_xalign() = 1;
 
@@ -136,13 +136,9 @@ namespace gnote {
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::clear_search_clicked));
     m_clear_search_button.show ();
 
-    m_case_sensitive.signal_toggled()
-      .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_case_sensitive_toggled));
-
     Gtk::Table *table = manage(new Gtk::Table (2, 3, false));
     table->attach (*label, 0, 1, 0, 1, Gtk::SHRINK, (Gtk::AttachOptions)0, 0, 0);
     table->attach (m_find_combo, 1, 2, 0, 1);
-    table->attach (m_case_sensitive, 1, 2, 1, 2);
     table->attach (m_clear_search_button,
                   2, 3, 0, 1,
                   Gtk::SHRINK, (Gtk::AttachOptions)0, 0, 0);
@@ -150,11 +146,9 @@ namespace gnote {
     table->show_all ();
 
     Gtk::HBox *hbox = manage(new Gtk::HBox (false, 2));
-    hbox->set_border_width(8);
-    hbox->pack_start (*image, false, false, 4);
     hbox->pack_start (*table, true, true, 0);
     hbox->show_all ();
-            
+
     // Notebooks Pane
     Gtk::Widget *notebooksPane = Gtk::manage(make_notebooks_pane ());
     notebooksPane->show ();
@@ -167,8 +161,8 @@ namespace gnote {
     m_status_bar.show();
 
     // Update on changes to notes
-    m.signal_note_deleted.connect(sigc::mem_fun(*this, &NoteRecentChanges::on_notes_changed));
-    m.signal_note_added.connect(sigc::mem_fun(*this, &NoteRecentChanges::on_notes_changed));
+    m.signal_note_deleted.connect(sigc::mem_fun(*this, &NoteRecentChanges::on_note_deleted));
+    m.signal_note_added.connect(sigc::mem_fun(*this, &NoteRecentChanges::on_note_added));
     m.signal_note_renamed.connect(sigc::mem_fun(*this, &NoteRecentChanges::on_note_renamed));
     m.signal_note_saved.connect(sigc::mem_fun(*this, &NoteRecentChanges::on_note_saved));
 
@@ -214,11 +208,41 @@ namespace gnote {
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_note_added_to_notebook));
     notebooks::NotebookManager::instance().signal_note_removed_from_notebook()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_note_removed_from_notebook));
+
+    // Set the focus chain for the top-most containers
+    std::vector<Gtk::Widget*> focus_chain;
+    focus_chain.push_back(hbox);
+    focus_chain.push_back(&m_hpaned);
+    vbox->set_focus_chain(focus_chain);
+
+    // Set focus chain for sub widgets of first top-most container
+    focus_chain.clear();
+    focus_chain.push_back(&m_find_combo);
+    focus_chain.push_back(&m_matches_window);
+    hbox->set_focus_chain(focus_chain);
+
+    // set focus chain for sub widgets of second top-most container
+    focus_chain.clear();
+    focus_chain.push_back(&m_matches_window);
+    focus_chain.push_back(notebooksPane);
+    m_hpaned.set_focus_chain(focus_chain);
+
+    // get back to the beginning of the focus chain
+    focus_chain.clear();
+    focus_chain.push_back(m_tree);
+    m_matches_window.set_focus_chain(focus_chain);
                         
     Gnote::obj().signal_quit.connect(sigc::mem_fun(*this, &NoteRecentChanges::on_exiting_event));
 
   }
 
+
+  NoteRecentChanges::~NoteRecentChanges()
+  {
+    Gnote::obj().remove_window(this);
+  }
+
+
   Gtk::MenuBar *NoteRecentChanges::create_menu_bar ()
   {
     ActionManager &am(ActionManager::obj());
@@ -384,6 +408,42 @@ namespace gnote {
     m_tree->append_column (*change);
   }
 
+  void NoteRecentChanges::add_note(const Note::Ptr & note)
+  {
+    std::string nice_date =
+      utils::get_pretty_print_date(note->change_date(), true);
+    Gtk::TreeIter iter = m_store->append();
+    iter->set_value(m_column_types.icon, s_note_icon);
+    iter->set_value(m_column_types.title, note->get_title());
+    iter->set_value(m_column_types.change_date, nice_date);
+    iter->set_value(m_column_types.note, note);
+  }
+
+  void NoteRecentChanges::delete_note(const Note::Ptr & note)
+  {
+    Gtk::TreeModel::Children rows = m_store->children();
+
+    for (Gtk::TreeModel::iterator iter = rows.begin();
+         rows.end() != iter; iter++) {
+      if (note == iter->get_value(m_column_types.note)) {
+        m_store->erase(iter);
+        break;
+      }
+    }
+  }
+
+  void NoteRecentChanges::rename_note(const Note::Ptr & note)
+  {
+    Gtk::TreeModel::Children rows = m_store->children();
+
+    for (Gtk::TreeModel::iterator iter = rows.begin();
+         rows.end() != iter; iter++) {
+      if (note == iter->get_value(m_column_types.note)) {
+        iter->set_value(m_column_types.title, note->get_title());
+        break;
+      }
+    }
+  }
 
   void NoteRecentChanges::update_results()
   {
@@ -481,8 +541,7 @@ namespace gnote {
         }
         return;
       }
-      if (!m_case_sensitive.get_active())
-        text = sharp::string_to_lower(text);
+      text = sharp::string_to_lower(text);
 
       m_current_matches.clear ();
       
@@ -492,17 +551,23 @@ namespace gnote {
         selected_notebook = notebooks::Notebook::Ptr();
       }
 
-      Search::ResultsPtr results =
-        search.search_notes(text, m_case_sensitive.get_active(), selected_notebook);
-      for(Search::Results::const_iterator iter = results->begin();
-          iter != results->end(); ++iter) {
-        m_current_matches[iter->first->uri()] = iter->second;
+      Search::ResultsPtr results = search.search_notes(text, false, selected_notebook);
+      // if no results found in current notebook ask user whether
+      // to search in all notebooks
+      if(results->size() == 0 && selected_notebook != NULL) {
+        no_matches_found_action();
       }
+      else {
+        for(Search::Results::const_reverse_iterator iter = results->rbegin();
+            iter != results->rend(); iter++) {
+          m_current_matches[iter->second->uri()] = iter->first;
+        }
       
-      add_matches_column ();
-      m_store_filter->refilter ();
-      m_tree->scroll_to_point (0, 0);
-      update_match_note_count (m_current_matches.size());
+        add_matches_column ();
+        m_store_filter->refilter ();
+        m_tree->scroll_to_point (0, 0);
+        update_match_note_count (m_current_matches.size());
+      }
   }
 
 
@@ -610,6 +675,49 @@ namespace gnote {
       ERR_OUT("exception: %s", e.what());
     }
   }
+
+  // called when no search results are found in the selected notebook
+  void NoteRecentChanges::no_matches_found_action()
+  {
+    m_hpaned.remove(m_matches_window);
+    if(!m_no_matches_box) {
+      Glib::ustring message = _("No results found in the selected notebook.\nClick here to search across all notes.");
+      Gtk::LinkButton *link_button = manage(new Gtk::LinkButton("", message));
+      link_button->signal_activate_link()
+        .connect(sigc::mem_fun(*this, &NoteRecentChanges::show_all_search_results));
+      link_button->set_tooltip_text(_("Click here to search across all notebooks"));
+      link_button->show();
+      Gtk::Table *no_matches_found_table = manage(new Gtk::Table(1, 3, false));
+      no_matches_found_table->attach(*link_button, 1, 2, 0, 1,
+        Gtk::FILL | Gtk::SHRINK,
+        Gtk::SHRINK,
+        0, 0
+      );
+
+      no_matches_found_table->set_col_spacings(4);
+      no_matches_found_table->show_all();
+      m_no_matches_box = manage(new Gtk::HBox(false, 0));
+      m_no_matches_box->pack_start(*no_matches_found_table, true, true, 0);
+      m_no_matches_box->show();
+    }
+    m_hpaned.add2(*m_no_matches_box);
+  }
+
+  void NoteRecentChanges::restore_matches_window()
+  {
+    if(m_no_matches_box && m_hpaned.get_child2() == m_no_matches_box) {
+      m_hpaned.remove(*m_no_matches_box);
+      m_hpaned.add2(m_matches_window);
+      restore_position();
+    }	
+  }
+
+  bool NoteRecentChanges::show_all_search_results()
+  {
+    Gtk::TreeIter iter = m_notebooksTree->get_model()->children().begin();
+    m_notebooksTree->get_selection()->select(iter);
+    return false;
+  }
                 
 
   /// <summary>
@@ -705,18 +813,28 @@ namespace gnote {
     perform_search ();
   }
 
-  void NoteRecentChanges::on_notes_changed(const Note::Ptr &)
+  void NoteRecentChanges::on_note_added(const Note::Ptr & note)
   {
-    update_results ();
+    restore_matches_window();
+    add_note(note);
   }
 
-  void NoteRecentChanges::on_note_renamed(const Note::Ptr&, const std::string&)
+  void NoteRecentChanges::on_note_deleted(const Note::Ptr & note)
   {
-    update_results ();
+    restore_matches_window();
+    delete_note(note);
+  }
+
+  void NoteRecentChanges::on_note_renamed(const Note::Ptr & note,
+                                          const std::string &)
+  {
+    restore_matches_window();
+    rename_note(note);
   }
 
   void NoteRecentChanges::on_note_saved(const Note::Ptr&)
   {
+    restore_matches_window();
     update_results ();
   }
 
@@ -783,7 +901,7 @@ namespace gnote {
       am ["OpenNoteAction"]->property_sensitive() = false;
       am ["DeleteNoteAction"]->property_sensitive() = false;
     } 
-    else if (selected_notes.size() > 1) {
+    else if (selected_notes.size()) {
       am ["OpenNoteAction"]->property_sensitive() = true;
       am ["DeleteNoteAction"]->property_sensitive() = true;
     } 
@@ -1079,7 +1197,9 @@ namespace gnote {
     // The following code has to be done for the MenuBar to
     // appear properly the next time this window is opened.
     if (m_menubar) {
-      m_content_vbox.remove (*m_menubar);
+      if(Gnote::obj().windowed()) {
+        m_content_vbox.remove (*m_menubar);
+      }
 #if 0
       am ["OpenNoteAction"].Activated -= OnOpenNote;
       am ["DeleteNoteAction"].Activated -= OnDeleteNote;
@@ -1095,10 +1215,9 @@ namespace gnote {
 //    Tomboy.ExitingEvent -= OnExitingEvent;
 
     hide ();
-    delete s_instance;
-    s_instance = NULL;
-    if (Gnote::obj().tray_icon_showing() == false) {
-      ActionManager::obj()["QuitGNoteAction"]->activate();
+    if(Gnote::obj().windowed()) {
+      delete s_instance;
+      s_instance = NULL;
     }
   }
 
@@ -1251,6 +1370,8 @@ namespace gnote {
       m_entry_changed_timeout->reset (500);
       m_clear_search_button.set_sensitive(true);
     }
+
+    restore_matches_window();
   }
 
   // Called in after .5 seconds of typing inactivity, or on
@@ -1272,16 +1393,11 @@ namespace gnote {
     // list, or shuffling an existing term to the top...
     bool repeat = false;
 
-    if (m_case_sensitive.get_active()) {
-      repeat = (find(s_previous_searches.begin(), s_previous_searches.end(), text) != s_previous_searches.end());
-    } 
-    else {
-      std::string lower = sharp::string_to_lower(text);
-      for(std::list<std::string>::const_iterator iter = s_previous_searches.begin();
-          iter != s_previous_searches.end(); ++iter) {
-        if (sharp::string_to_lower(*iter) == lower) {
-          repeat = true;
-        }
+    std::string lower = sharp::string_to_lower(text);
+    for(std::list<std::string>::const_iterator iter = s_previous_searches.begin();
+        iter != s_previous_searches.end(); ++iter) {
+      if (sharp::string_to_lower(*iter) == lower) {
+        repeat = true;
       }
     }
 
@@ -1347,6 +1463,7 @@ namespace gnote {
 
   void NoteRecentChanges::on_notebook_selection_changed()
   {
+    restore_matches_window();
     notebooks::Notebook::Ptr notebook = get_selected_notebook ();
     ActionManager & am(ActionManager::obj());
     if (!notebook) {
@@ -1525,12 +1642,14 @@ namespace gnote {
   void NoteRecentChanges::on_note_added_to_notebook (const Note & , 
                                                      const notebooks::Notebook::Ptr & )
   {
+    restore_matches_window();
     update_results ();
   }
     
   void NoteRecentChanges::on_note_removed_from_notebook (const Note & , 
                                                        const notebooks::Notebook::Ptr & )
   {
+    restore_matches_window();
     update_results ();
   }
 
