@@ -33,7 +33,7 @@
 #include <gdkmm/pixbuf.h>
 #include <gtkmm/cellrenderer.h>
 #include <gtkmm/checkbutton.h>
-#include <gtkmm/comboboxentry.h>
+#include <gtkmm/combobox.h>
 #include <gtkmm/liststore.h>
 #include <gtkmm/menubar.h>
 #include <gtkmm/paned.h>
@@ -63,6 +63,7 @@ public:
   static NoteRecentChanges *get_instance();
   static NoteRecentChanges *get_instance(NoteManager& m);
 
+  virtual ~NoteRecentChanges();
   void set_search_text(const std::string & value);
 
 //////
@@ -90,6 +91,9 @@ private:
   void matches_column_data_func(Gtk::CellRenderer *, const Gtk::TreeIter &);
   void update_total_note_count (int);
   void update_match_note_count (int);
+  void no_matches_found_action();
+  void restore_matches_window();
+  bool show_all_search_results();
   bool filter_notes(const Gtk::TreeIter & );
   bool filter_tags(const Gtk::TreeIter & );
   bool filter_by_tag (const Note::Ptr &);
@@ -142,7 +146,7 @@ private:
   std::string get_search_text();
   void save_position ();
   void restore_position();
-  bool on_exiting_event();
+  void on_exiting_event();
 
   class RecentNotesColumnTypes
     : public Gtk::TreeModelColumnRecord
@@ -166,23 +170,24 @@ private:
       {
         add(text);
       }
-    Gtk::TreeModelColumn<std::string> text;
+    Gtk::TreeModelColumn<Glib::ustring> text;
   };
 
   NoteManager &       m_manager;
   Gtk::MenuBar       *m_menubar;
   RecentSearchColumnTypes m_find_combo_columns;
-  Gtk::ComboBoxEntry  m_find_combo;
+  Gtk::ComboBox       m_find_combo;
   Gtk::Button         m_clear_search_button;
-  Gtk::CheckButton    m_case_sensitive;
   Gtk::Statusbar      m_status_bar;
   Gtk::ScrolledWindow m_matches_window;
   Gtk::HPaned         m_hpaned;
   Gtk::VBox           m_content_vbox;
   Gtk::TreeViewColumn *m_matches_column;
-        
+
+  Gtk::HBox           *m_no_matches_box;
+
   notebooks::NotebooksTreeView  *m_notebooksTree;
-        
+
   // Use the following like a Set
   std::set<Tag::Ptr>  m_selected_tags;
 
