@@ -762,20 +762,22 @@ namespace gnote {
   {
     // Some of these checks should be replaced with fixes to
     // TitleTrie.FindMatches, probably.
-    if (hit.value.expired()) {
-      DBG_OUT("DoHighlight: null pointer error for '%s'." , hit.key.c_str());
+    if (hit.value().expired()) {
+      DBG_OUT("DoHighlight: null pointer error for '%s'." , hit.key().c_str());
       return;
     }
       
-    if (!manager().find(hit.key)) {
-      DBG_OUT ("DoHighlight: '%s' links to non-existing note." , hit.key.c_str());
+    if (!manager().find(hit.key())) {
+      DBG_OUT ("DoHighlight: '%s' links to non-existing note." ,
+               hit.key().c_str());
       return;
     }
       
-    Note::Ptr hit_note(hit.value);
+    Note::Ptr hit_note(hit.value());
 
-    if (sharp::string_to_lower(hit.key) != sharp::string_to_lower(hit_note->get_title())) { // == 0 if same string  
-      DBG_OUT ("DoHighlight: '%s' links wrongly to note '%s'." , hit.key.c_str(), 
+    if (sharp::string_to_lower(hit.key()) != sharp::string_to_lower(hit_note->get_title())) { // == 0 if same string
+      DBG_OUT ("DoHighlight: '%s' links wrongly to note '%s'." ,
+               hit.key().c_str(),
                hit_note->get_title().c_str());
       return;
     }
@@ -784,10 +786,10 @@ namespace gnote {
       return;
 
     Gtk::TextIter title_start = start;
-    title_start.forward_chars (hit.start);
+    title_start.forward_chars (hit.start());
 
     Gtk::TextIter title_end = start;
-    title_end.forward_chars (hit.end);
+    title_end.forward_chars (hit.end());
 
     // Only link against whole words/phrases
     if ((!title_start.starts_word () && !title_start.starts_sentence ()) ||
@@ -801,7 +803,7 @@ namespace gnote {
     }
 
     DBG_OUT ("Matching Note title '%s' at %d-%d...",
-             hit.key.c_str(), hit.start, hit.end);
+             hit.key().c_str(), hit.start(), hit.end());
 
     get_buffer()->remove_tag (m_broken_link_tag, title_start, title_end);
     get_buffer()->apply_tag (m_link_tag, title_start, title_end);
@@ -906,6 +908,12 @@ namespace gnote {
       }
     }
 
+    Glib::RefPtr<Gtk::TextTag> broken_link_tag = get_note()->get_tag_table()->get_broken_link_tag();
+    if(start.begins_tag(broken_link_tag)) {
+      get_note()->get_buffer()->remove_tag(broken_link_tag, start, end);
+      get_note()->get_buffer()->apply_tag(get_note()->get_tag_table()->get_link_tag(), start, end);
+    }
+
     // FIXME: We used to also check here for (link != this.Note), but
     // somehow this was causing problems receiving clicks for the
     // wrong instance of a note (see bug #413234).  Since a
@@ -1264,8 +1272,8 @@ namespace gnote {
   void NoteTagsWatcher::on_tag_removed(const Note::Ptr&, const std::string& tag_name)
   {
     Tag::Ptr tag = TagManager::obj().get_tag (tag_name);
-    DBG_OUT ("Watchers.OnTagRemoved popularity count: %d", tag->popularity());
-    if (tag->popularity() == 0) {
+    DBG_OUT ("Watchers.OnTagRemoved popularity count: %d", tag ? tag->popularity() : 0);
+    if (tag && tag->popularity() == 0) {
       TagManager::obj().remove_tag (tag);
     }
   }
